@@ -29,21 +29,32 @@
 #include <glib/gi18n.h>
 #include <stdlib.h>
 
+#ifdef GDK_WINDOWING_WAYLAND
+#include <gdk/wayland/gdkwayland.h>
+#endif
+
 typedef struct
 {
 	int level;
 	guint keyval;
 } KeyInfo;
 
+typedef struct
+{
+	GtkWindow *window;
+	TeclaView *view;
+	TeclaModel *model;
+	gulong remove_handler_id;
+} TeclaInstance;
+
 struct _TeclaApplication
 {
 	GtkApplication parent_instance;
-	GtkWindow *main_window;
-	TeclaView *main_view;
-	TeclaModel *main_model;
 	TeclaKeymapObserver *observer;
+	TeclaInstance main;
+	GList *instances; /* TeclaInstance* */
 	gchar *layout;
-	gboolean ignore_first_change;
+	gchar *parent_handle;
 };
 
 static GtkPopover *current_popover = NULL;
@@ -55,14 +66,17 @@ tecla_application_command_line (GApplication            *app,
 				GApplicationCommandLine *cl)
 {
 	TeclaApplication *tecla_app = TECLA_APPLICATION (app);
+	GVariantDict *options;
 	g_autofree GStrv argv = NULL;
 	int argc;
 
+	options = g_application_command_line_get_options_dict (cl);
 	argv = g_application_command_line_get_arguments (cl, &argc);
 
 	if (argc > 1) {
-		g_free (tecla_app->layout);
-		tecla_app->layout = g_strdup (argv[1]);
+		g_set_str (&tecla_app->layout, argv[1]);
+		g_set_str (&tecla_app->parent_handle, NULL);
+		g_variant_dict_lookup (options, "parent-handle", "s", &tecla_app->parent_handle);
 	}
 
 	g_application_activate (app);
@@ -71,13 +85,14 @@ tecla_application_command_line (GApplication            *app,
 }
 
 const GOptionEntry all_options[] = {
+	{ "parent-handle", 0, 0, G_OPTION_ARG_STRING, NULL, N_("Attach to a parent window"), N_("Window handle") },
 	{ "version", 0, 0, G_OPTION_ARG_NONE, NULL, N_("Display version number"), NULL },
 	{ NULL, 0, 0, 0, NULL, NULL, NULL } /* end the list */
 };
 
 static int
 tecla_application_handle_local_options (GApplication *app,
-		      GVariantDict *options)
+					GVariantDict *options)
 {
 	if (g_variant_dict_contains (options, "version")) {
 		g_print ("%s %s\n", PACKAGE, VERSION);
@@ -347,18 +362,13 @@ observer_keymap_notify_cb (TeclaKeymapObserver *observer,
 	g_autoptr (TeclaModel) model = NULL;
 	struct xkb_keymap *xkb_keymap;
 
-	if (app->ignore_first_change) {
-		app->ignore_first_change = FALSE;
-		return;
-	}
-
 	xkb_keymap = tecla_keymap_observer_get_keymap (observer);
 	model = tecla_model_new_from_xkb_keymap (xkb_keymap);
-	connect_model (app->main_window,
-		       app->main_view, model);
-	update_title (app->main_window, model);
+	connect_model (app->main.window,
+		       app->main.view, model);
+	update_title (app->main.window, model);
 
-	g_set_object (&app->main_model, model);
+	g_set_object (&app->main.model, model);
 }
 
 static void
@@ -369,43 +379,102 @@ observer_keymap_group_cb (TeclaKeymapObserver *observer,
 	int group;
 
 	group = tecla_keymap_observer_get_group (observer);
-	if (app->main_model)
-		tecla_model_set_group (app->main_model, group);
+	if (app->main.model)
+		tecla_model_set_group (app->main.model, group);
+}
+
+void
+window_removed_cb (TeclaApplication *tecla_app,
+                   GtkWindow        *window,
+                   gpointer          user_data)
+{
+	TeclaInstance *instance = user_data;
+
+	tecla_app->instances =
+		g_list_remove (tecla_app->instances, instance);
+
+	g_signal_handler_disconnect (tecla_app, instance->remove_handler_id);
+
+	g_clear_object (&instance->model);
+	g_free (instance);
+}
+
+void
+main_window_removed_cb (TeclaApplication *tecla_app,
+                        GtkWindow        *window,
+                        gpointer          user_data)
+{
+	if (tecla_app->main.window != window)
+		return;
+
+	g_clear_object (&tecla_app->observer);
+	g_clear_object (&tecla_app->main.model);
+	tecla_app->main.view = NULL;
+	tecla_app->main.window = NULL;
 }
 
 static void
 tecla_application_activate (GApplication *app)
 {
 	TeclaApplication *tecla_app = TECLA_APPLICATION (app);
+	g_autofree char *layout = g_steal_pointer (&tecla_app->layout);
+	g_autofree char *parent_handle = g_steal_pointer (&tecla_app->parent_handle);
 
-	if (!tecla_app->main_window) {
-		tecla_app->main_window =
-			create_window (tecla_app, &tecla_app->main_view);
-
-		tecla_app->ignore_first_change = tecla_app->layout != NULL;
-
-		tecla_app->observer = tecla_keymap_observer_new ();
-		g_signal_connect (tecla_app->observer, "notify::keymap",
-				  G_CALLBACK (observer_keymap_notify_cb), app);
-		g_signal_connect (tecla_app->observer, "notify::group",
-				  G_CALLBACK (observer_keymap_group_cb), app);
-	}
-
-	if (tecla_app->layout) {
-		g_clear_object (&tecla_app->main_model);
-		tecla_app->main_model =
-			tecla_model_new_from_layout_name (tecla_app->layout);
-
-		if (tecla_app->main_model) {
-			connect_model (tecla_app->main_window,
-				       tecla_app->main_view,
-				       tecla_app->main_model);
-			g_clear_pointer (&tecla_app->layout, g_free);
-			update_title (tecla_app->main_window, tecla_app->main_model);
+	if (!layout) {
+		if (!tecla_app->main.window) {
+			tecla_app->main.window =
+				create_window (tecla_app, &tecla_app->main.view);
+			g_signal_connect (tecla_app, "window-removed",
+					  G_CALLBACK (main_window_removed_cb),
+					  NULL);
 		}
-	}
 
-	gtk_window_present (tecla_app->main_window);
+		if (!tecla_app->observer) {
+			tecla_app->observer = tecla_keymap_observer_new ();
+			g_signal_connect (tecla_app->observer, "notify::keymap",
+					  G_CALLBACK (observer_keymap_notify_cb), app);
+			g_signal_connect (tecla_app->observer, "notify::group",
+					  G_CALLBACK (observer_keymap_group_cb), app);
+		}
+
+		gtk_window_present (tecla_app->main.window);
+	} else {
+		TeclaInstance *instance = g_new0 (TeclaInstance, 1);
+
+		instance->window = create_window (tecla_app, &instance->view);
+
+		instance->model =
+			tecla_model_new_from_layout_name (layout);
+
+		if (instance->model) {
+			connect_model (instance->window,
+				       instance->view,
+				       instance->model);
+			update_title (instance->window, instance->model);
+		}
+
+#ifdef GDK_WINDOWING_WAYLAND
+		if (parent_handle &&
+		    GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (GTK_WIDGET (instance->window)))) {
+			GdkSurface *surface;
+
+			gtk_widget_realize (GTK_WIDGET (instance->window));
+			surface = gtk_native_get_surface (GTK_NATIVE (instance->window));
+			gdk_wayland_toplevel_set_transient_for_exported (GDK_TOPLEVEL (surface),
+									 parent_handle);
+		}
+#endif
+
+		instance->remove_handler_id =
+			g_signal_connect (tecla_app, "window-removed",
+					  G_CALLBACK (window_removed_cb),
+					  instance);
+
+		tecla_app->instances =
+			g_list_prepend (tecla_app->instances, instance);
+
+		gtk_window_present (instance->window);
+	}
 }
 
 static void
@@ -421,6 +490,7 @@ tecla_application_class_init (TeclaApplicationClass *klass)
 static void
 tecla_application_init (TeclaApplication *app)
 {
+	gtk_window_set_default_icon_name ("org.gnome.Tecla");
 	g_application_add_main_option_entries (G_APPLICATION (app), all_options);
 }
 
