@@ -95,6 +95,52 @@ tecla_model_init (TeclaModel *model)
 {
 }
 
+static struct {
+        gunichar ch;
+        const char *nick;
+} notable_chars[] = {
+        { 0x00a, "⍽"      }, /* NO-BREAK SPACE */
+        { 0x00ad, "SHY"   }, /* SOFT HYPHEN */
+        { 0x034f, "CGJ"   }, /* COMBINING GRAPHEME JOINER */
+        { 0x061c, "ALM"   }, /* ARABIC LETTER MARK */
+        { 0x200b, "ZWS"   }, /* ZERO WIDTH SPACE */
+        { 0x200c, "ZWNJ"  }, /* ZERO WIDTH NON-JOINER */
+        { 0x200d, "ZWJ"   }, /* ZERO WIDTH JOINER */
+        { 0x200e, "LRM"   }, /* LEFT-TO-RIGHT MARK */
+        { 0x200f, "RLM"   }, /* RIGHT-TO-LEFT MARK */
+        { 0x2028, "LS"    }, /* LINE SEPARATOR */
+        { 0x2029, "PS"    }, /* PARAGRAPH SEPARATOR */
+        { 0x202a, "LRE"   }, /* LEFT-TO-RIGHT EMBEDDING */
+        { 0x202b, "RLE"   }, /* RIGHT-TO-LEFT EMBEDDING */
+        { 0x202c, "PDF"   }, /* POP DIRECTIONAL FORMATTING */
+        { 0x202d, "LRO"   }, /* LEFT-TO-RIGHT OVERRIDE */
+        { 0x202e, "RLO"   }, /* RIGHT-TO-LEFT OVERRIDE */
+        { 0x202f, "⍽"     }, /* NARROW NO-BREAK SPACE */
+        { 0x2060, "WJ"    }, /* WORD JOINER */
+        { 0x2061, "FA"    }, /* FUNCTION APPLICATION */
+        { 0x2062, "IT"    }, /* INVISIBLE TIMES */
+        { 0x2063, "IS"    }, /* INVISIBLE SEPARATOR */
+        { 0x2066, "LRI"   }, /* LEFT-TO-RIGHT ISOLATE */
+        { 0x2067, "RLI"   }, /* RIGHT-TO-LEFT ISOLATE */
+        { 0x2068, "FSI"   }, /* FIRST STRONG ISOLATE */
+        { 0x2069, "PDI"   }, /* POP DIRECTIONAL ISOLATE */
+        { 0xfeff, "ZWNBS" }, /* ZERO WIDTH NO-BREAK SPACE */
+};
+
+static const gchar *
+get_unicode_nick (gunichar ch)
+{
+        for (gsize i = 0; i < G_N_ELEMENTS (notable_chars); i++) {
+                if (ch < notable_chars[i].ch)
+                        return NULL;
+
+                if (ch == notable_chars[i].ch)
+                        return notable_chars[i].nick;
+        }
+
+        return NULL;
+}
+
 static gchar *
 get_key_label (xkb_keysym_t key)
 {
@@ -108,8 +154,12 @@ get_key_label (xkb_keysym_t key)
 		label = "";
 		break;
 
+	case GDK_KEY_Delete:
+		label = "⌦";
+		break;
+
 	case GDK_KEY_BackSpace:
-		label = "";
+		label = "⌫";
 		break;
 
 	case GDK_KEY_space:
@@ -117,91 +167,184 @@ get_key_label (xkb_keysym_t key)
 		break;
 
 	case GDK_KEY_dead_grave:
-		label = "ˋ";
+		label = "◌̀";
 		break;
 
 	case GDK_KEY_dead_abovecomma:
-		label = "̓";
+		label = "̓◌̓";
 		break;
 
 	case GDK_KEY_dead_abovereversedcomma:
-		label = "̔";
+		label = "̔◌̔";
 		break;
 
 	case GDK_KEY_dead_acute:
-		label = "ˊ";
+		label = "◌́";
 		break;
 
 	case GDK_KEY_dead_circumflex:
-		label = "ˆ";
+		label = "◌̂";
 		break;
 
 	case GDK_KEY_dead_tilde:
-		label = "~";
+		label = "◌̃";
 		break;
 
 	case GDK_KEY_dead_macron:
-		label = "ˉ";
+		label = "◌̄";
 		break;
 
 	case GDK_KEY_dead_breve:
-		label = "˘";
+		label = "◌̆";
 		break;
 
 	case GDK_KEY_dead_abovedot:
-		label = "˙";
+		label = "◌̇";
 		break;
 
 	case GDK_KEY_dead_diaeresis:
-		label = "¨";
+		label = "◌̈";
 		break;
 
 	case GDK_KEY_dead_abovering:
-		label = "˚";
+		label = "◌̊";
 		break;
 
 	case GDK_KEY_dead_doubleacute:
-		label = "˝";
+		label = "◌̋";
 		break;
 
 	case GDK_KEY_dead_caron:
-		label = "ˇ";
+		label = "◌̌";
 		break;
 
 	case GDK_KEY_dead_cedilla:
-		label = "¸";
+		label = "◌̧";
 		break;
 
 	case GDK_KEY_dead_ogonek:
-		label = "˛";
+		label = "◌̨";
 		break;
 
 	case GDK_KEY_dead_belowdot:
-		label = " ̣";
+		label = "◌̣";
 		break;
 
 	case GDK_KEY_dead_hook:
-		label = "̉";
+		label = "◌̉";
 		break;
 
 	case GDK_KEY_dead_horn:
-		label = "̛";
+		label = "◌̛";
 		break;
 
 	case GDK_KEY_dead_stroke:
-		label = "̵";
+		label = "◌̵ ";
+		break;
+
+	case GDK_KEY_dead_hamza:
+		label = "ء";
 		break;
 
 	case GDK_KEY_horizconnector:
 		label = "";
 		break;
 
+	case GDK_KEY_dead_belowcomma:
+		label = "◌̦";
+		break;
+
+	case GDK_KEY_dead_iota:
+		label = "◌ͅ";
+		break;
+
+	case GDK_KEY_dead_doublegrave:
+		label = "◌̏";
+		break;
+
+	case GDK_KEY_dead_belowring:
+		label = "◌̥";
+		break;
+
+	case GDK_KEY_dead_belowmacron:
+		label = "◌̱";
+		break;
+
+	case GDK_KEY_dead_belowcircumflex:
+		label = "◌̭";
+		break;
+
+	case GDK_KEY_dead_belowtilde:
+		label = "◌̰";
+		break;
+
+	case GDK_KEY_dead_belowbreve:
+		label = "◌̮";
+		break;
+
+	case GDK_KEY_dead_belowdiaeresis:
+		label = "◌̤";
+		break;
+
+	case GDK_KEY_dead_lowline:
+		label = "◌̲";
+		break;
+
+	case GDK_KEY_dead_aboveverticalline:
+		label = "◌̍ ";
+		break;
+
+	case GDK_KEY_dead_belowverticalline:
+		label = "◌̩";
+		break;
+
+	case GDK_KEY_dead_longsolidusoverlay:
+		label = "◌̸ ";
+		break;
+
+	case GDK_KEY_dead_voiced_sound:
+		label = "◌゙";
+		break;
+
+	case GDK_KEY_dead_a:
+		label = "◌ͣ";
+		break;
+
+	case GDK_KEY_dead_e:
+		label = "◌ͤ";
+		break;
+
+	case GDK_KEY_dead_i:
+		label = "◌ͥ";
+		break;
+
+	case GDK_KEY_dead_o:
+		label = "◌ͦ";
+		break;
+
+	case GDK_KEY_dead_u:
+		label = "◌ͧ";
+		break;
+
+	case GDK_KEY_dead_small_schwa:
+		label = "◌ᷪ";
+		break;
+
+	case GDK_KEY_dead_greek:
+		label = "a→α";
+		break;
+
+	case GDK_KEY_dead_currency:
+		label = "e→€";
+		break;
+
 	case GDK_KEY_Multi_key:
 		label = "";
 		break;
 
+	case GDK_KEY_ISO_Enter:
 	case GDK_KEY_Return:
-		label = "";
+		label = "⏎";
 		break;
 
 	case GDK_KEY_Shift_L:
@@ -215,7 +358,7 @@ get_key_label (xkb_keysym_t key)
 
 	case GDK_KEY_Tab:
 	case GDK_KEY_ISO_Left_Tab:
-		label = "";
+		label = "⭾";
 		break;
 
 	case GDK_KEY_Alt_L:
@@ -257,9 +400,13 @@ get_key_label (xkb_keysym_t key)
 			buf[g_unichar_to_utf8 (uc, buf)] = '\0';
 			return g_strdup (buf);
 		} else {
+                        const gchar *nick = get_unicode_nick (uc);
 			const gchar *name = gdk_keyval_name (key);
 
-			if (name) {
+                        if (nick) {
+                                label = nick;
+                        }
+			else if (name) {
 				g_autofree gchar *fixed_name = NULL;
 				gchar *p;
 
@@ -310,6 +457,10 @@ tecla_model_new_from_layout_name (const gchar *name)
 	};
 
 	sep = strchr (name, '+');
+	if (!sep)
+		sep = strchr (name, ' ');
+	if (!sep)
+		sep = strchr (name, '\t');
 
 	if (sep) {
 		variant = sep + 1;
